﻿using SciChart.Charting.ChartModifiers;
using SciChart.Core.Utility.Mouse;
using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Input;

namespace SciChartSample
{
    public class DataPointSelectionModifierExtention : DataPointSelectionModifier
    {
        /// <summary>
        /// https://www.scichart.com/questions/wpf/i-want-to-bind-selectedpointmarkers-of-datapointselectionmodifier
        /// </summary>
        public ObservableCollection<DataPointInfo> SelectedPointMarkers2
        {
            get => (ObservableCollection<DataPointInfo>)GetValue(SelectedPointMakers2Property);
            set => SetValue(SelectedPointMakers2Property, value);
        }
        public static readonly DependencyProperty SelectedPointMakers2Property =
            DependencyProperty.Register(
                nameof(SelectedPointMarkers2),
                typeof(ObservableCollection<DataPointInfo>),
                typeof(DataPointSelectionModifierExtention),
                new PropertyMetadata(new ObservableCollection<DataPointInfo>()));

        public DataPointSelectionModifierExtention()
        {
            base.SelectedPointMarkers.CollectionChanged += SelectedPointMakers_CollectionChanged;
        }

        private void SelectedPointMakers_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    foreach (var item in e.NewItems.Cast<DataPointInfo>())
                    {
                        this.SelectedPointMarkers2.Add(item);
                    }
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Reset:
                    SelectedPointMarkers2.Clear();
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                    foreach (var item in e.OldItems.Cast<DataPointInfo>())
                    {
                        this.SelectedPointMarkers2.Remove(item);
                    }
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Replace:
                case System.Collections.Specialized.NotifyCollectionChangedAction.Move:
                default:
                    throw new ApplicationException();
            }
        }

        public override void OnModifierMouseDown(ModifierMouseArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Shift)
            {
                DeselectAll();
                base.OnModifierMouseDown(e);
            }
            else if (Keyboard.Modifiers == ModifierKeys.Control)
            {
                base.OnModifierMouseDown(e);
            }
            else
            {
                e.Handled = false;
                DeselectAll();
            }
        }

        public override void OnModifierMouseUp(ModifierMouseArgs e)
        {
            base.OnModifierMouseUp(e);
            this.UpdateState();
        }
        
        public void DeselectAll()
        {
            this.DeselectAllPointMarkers();
            this.UpdateState();
        }
    }
}